#include <iostream>
#include <vector>
#include <algorithm>
#include <limits>
#include <deque>
#include <cmath>
#include <cassert>
using namespace std;

typedef vector<double> VD;
typedef vector<VD> VVD;

typedef vector<int> VI;
typedef vector<VI> VVI;

VVD p;
VVI m;  // 1 for open space
VVI scratch;
int X, Y, sigma;
const double INF = std::numeric_limits<double>::infinity();

int dx[] = { 0, 1, 0,-1};
int dy[] = {-1, 0, 1, 0};
char codes[] = {'N', 'E', 'S', 'W'};

struct node {int dist; int x; int y;};
double exp_dist(int x, int y)
{
  node start;
  start.dist = 0; start.x = x; start.y = y;
  deque<node> q;
  q.push_front(start);

  for(int xx = 0; xx < X; xx++)
    for(int yy = 0; yy < Y; yy++)
      scratch[xx][yy] = 0;
  scratch[x][y] = 1;

  double ret = 0;

  while(!q.empty())
    {
      node n = q.front();
      q.pop_front();

      ret += n.dist * p[n.x][n.y];
      for(int i = 0; i < 4; i++)
	if (!scratch[n.x+dx[i]][n.y+dy[i]] && m[n.x+dx[i]][n.y+dy[i]])
	  {
	    scratch[n.x+dx[i]][n.y+dy[i]] = 1;
	    node n2;
	    n2.dist = n.dist + 1;
	    n2.x = n.x+dx[i];
	    n2.y = n.y+dy[i];
	    q.push_back(n2);
	  }
    }

  return ret;
}

void update_step()
{
  VVD new_p(X, VD(Y, 0));
  for(int x = 0; x < X; x++)
    for(int y = 0; y < Y; y++)
      {
	/* wrong:
	if (!m[x][y])
	  continue;
	for(int i = 0; i < 4; i++)
	    p[x][y] += p[x+dx[i]][y+dy[i]] / 4;
	*/

	if (!m[x][y])
	  continue;
	int denom = 0;
	for(int i = 0; i < 4; i++)
	  if (m[x+dx[i]][y+dy[i]])
	    denom++;
	for(int i = 0; i < 4; i++)
	  if (m[x+dx[i]][y+dy[i]])
	    new_p[x+dx[i]][y+dy[i]] += p[x][y] / denom;
      }

  swap(p, new_p);

#if 0
  for(int yy = 0; yy < Y; yy++)
    {
      for(int xx = 0; xx < X; xx++)
	{
	  printf("%03d ", (int)(p[xx][yy]*1000));
	}
      printf("\n");
    }
#endif
}

void update_obs(int x, int y)
{
  double sum = 0;
  for(int xx = 0; xx < X; xx++)
    for(int yy = 0; yy < Y; yy++)
      {
	double dist_squared = double(xx-x)*(xx-x) + double(yy-y)*(yy-y);
	p[xx][yy] *= exp(-dist_squared / (2*sigma*sigma));
	sum += p[xx][yy];
      }

  for(int yy = 0; yy < Y; yy++)
    {
      for(int xx = 0; xx < X; xx++)
	{
	  p[xx][yy] *= (1/sum);
	  //	  printf("%.2f ", p[xx][yy]);
	}
      //      printf("\n");
    }
}

struct dir
{
  int dx, dy;
  double edist;
  int prio;
  char code;
  bool operator < (const dir &rhs) const
  {
    return edist < rhs.edist;
  }
};

int main()
{
  cin >> X >> Y;
  p = VVD(X, VD(Y, 0));
  m = VVI(X, VI(Y));
  int hx = -1, hy = -1;
  scratch = m;
  for(int y = 0; y < Y; y++)
    {
      string s;
      cin >> s;
      for(int x = 0; x < X; x++)
	{
	  m[x][y] = (s[x] == '#' ? 0 : 1);
	  if (s[x] == 'h')
	    {
	      assert(hx == -1);
	      hx = x;
	      hy = y;
	    }
	  p[x][y] = 1.0/(X*Y);  // It'll get normalized later.
	}
    }

  assert(hx != -1);

  cin >> sigma;

  while(true)
    {
      string s;
      cin >> s;
      if (!cin)
	return 0;
      update_step();
      if (s != "silence")
	{
	  int sound_x = atoi(s.c_str());
	  int sound_y;
	  cin >> sound_y;
	  update_obs(sound_x, sound_y);
	}

      dir dirs[5];
      dirs[0].dx = dirs[0].dy = 0;
      dirs[0].edist = exp_dist(hx, hy);
      dirs[0].prio = 0;
      dirs[0].code = '.';
      for(int i = 0; i < 4; i++)
	{
	  dirs[i+1].dx = dx[i];
	  dirs[i+1].dy = dy[i];
	  if (!m[hx+dx[i]][hy+dy[i]])
	    dirs[i+1].edist = INF;
	  else
	    dirs[i+1].edist = exp_dist(hx+dx[i], hy+dy[i]);
	  dirs[i+1].prio = i+1;
	  dirs[i+1].code = codes[i];
	}

      int best = 4;
      for(int i = 3; i >= 0; i--)
	{
	  if (dirs[i].edist < dirs[best].edist + 1e-5)
	    best = i;
	}

      //      for(int i = 0; i < 5; i++)
      //	{
      //	  printf("code = %c; cost = %f\n", dirs[i].code, dirs[i].edist);
      //	}
      //      printf("best = %c hx=%d hy=%d\n", dirs[best].code, hx, hy);

      hx += dirs[best].dx;
      hy += dirs[best].dy;
      cout << dirs[best].code << endl;

      //      printf("%d %d\n", hx, hy);
    }
}
