#include <cstdio>
#include <algorithm>
#include <map>
using namespace std;

const int MAXN = 100001;

const long long MOD = 1000003;

struct Edge {
  int a, b, w;
  
  Edge() {}	
  Edge(int _a, int _b, int _w) : a(_a), b(_b), w(_w) {}
  
  friend bool operator<(const Edge &a, const Edge &b) {
    return a.w < b.w;
  }
};


Edge e[MAXN];
int n, m;

long long ret;

bool root[MAXN];
int o[MAXN];


void init_union_find() {
  for (int i = 0; i < MAXN; i++) {
    root[i] = true;
    o[i] = 1;
  }
}

int group(int x) {
  int tmp = x, tmp1;
  while (!root[x]) 
    x = o[x];
  while (!root[tmp]) {
    tmp1 = o[tmp];
    o[tmp] = x;
    tmp = tmp1;
  }
  return x;
}

void merge(int xx, int yy) {
  int x = group(xx), y = group(yy);
  if (x == y)
    return;
  
  if (o[x] > o[y]) {
    o[x] += o[y];
    o[y] = x;
    root[y] = false;
  }
  else {
    o[y] += o[x];
    o[x] = y;
    root[x] = false;
  }
}

void count2(int num) {
  if (num == 2)
    ret *= 2;
}

void count3(int *a, int *b, int num) {
  if (num == 2)
    ret *= 3;
  else if (num == 3) {
    if ((a[0] == a[1] && b[0] == b[1]) || (a[0] == a[2] && b[0] == b[2]) || (a[1] == a[2] && b[1] == b[2]))
      ret *= 2;
    else
      ret *= 3;
  }
  else if (num == 4) {
    if ((a[0] == a[1] && b[0] == b[1]) || (a[0] == a[2] && b[0] == b[2]) || (a[1] == a[2] && b[1] == b[2])) ret *= 2;
  }
}

void count4(int *a, int *b, int num) {
  bool t = false;
  for (int i = 0; !t && i < 4; i++)
    for (int j = i + 1; !t && j < 4; j++)
      if (a[i] == a[j] && b[i] == b[j])
	t = true;
  
  bool z = false;
  for (int i = 0; !z && i < 4; i++) {
    z = true;
    for (int j = 0; z && j < 4; j++) {
      if (i == j)
	continue;
      if (a[i] == a[j] || a[i] == b[j] || b[i] == a[j] || b[i] == b[j])
	z = false;
    }
  }
  
  int v[8];
  memset(v, 0, sizeof(v));
  for (int i = 0; i < 4; i++) {
    v[a[i]-1]++;
    v[b[i]-1]++;
  }
  sort(&v[0], &v[num]);
  
  if (num == 2)
    ret *= 4;
  else if (num == 3) {
    if (v[0] == 2 && v[1] == 2 && v[2] == 4)
      ret *= 4;
    else if (v[0] == 1 && v[1] == 3 && v[2] == 4)
      ret *= 3;
    else
      ret *= 5;
  }
  else if (num == 4) {
    if (v[0] == 2 && v[1] == 2 && v[2] == 2 && v[3] == 2)
      ret *= 4;
    else if
      (t && z)
      ret *= 3;
    else {
      if (t)
	ret *= 2;
      else if (v[0] == 1 && v[1] == 2 && v[2] == 2 && v[3] == 3)
	ret *= 3;
    }
  }
  else if (num == 5) {
    if (t)
      ret *= 2;
    else if (z)
      ret *= 3;
  }
  else if (num == 6) {
    if (t)
      ret *= 2;
  }
}


void count(int *p, int c) {
  if (c <= 1)
    return;
  else {
    map<int, int> M;
    int num = 1;
    int a[4], b[4];
    
    for (int i = 0; i < c; i++) {
      int g1 = group(e[p[i]].a);
      int g2 = group(e[p[i]].b);
      if (M[g1] == 0)
	M[g1] = num++;
      if (M[g2] == 0)
	M[g2] = num++;
      a[i] = M[g1], b[i] = M[g2];
      if (a[i] > b[i]) 
	swap(a[i], b[i]);
    }
    
    num--;
    
    if (c == 4)
      count4(a, b, num);
    else if (c == 3)
      count3(a, b, num);
    else if (c == 2)
      count2(num);
    
    ret %= MOD;
  }
}

void kruskal() {
  init_union_find();
  sort(&e[0], &e[m]);
  
  ret = 1;
	
  int last = e[0].w;
  
  int nextGroup[4];
  int c = 0;
  
  nextGroup[c++] = 0;
  
  for (int i = 1; i < m; i++) {
    if (e[i].w != last) {
      count(nextGroup, c);
      
      for (int j = 0; j < c; j++)
	merge(e[nextGroup[j]].a, e[nextGroup[j]].b);
      
      c = 0;
      last = e[i].w;      
    }
    
    int g1 = group(e[i].a);
    int g2 = group(e[i].b);
    
    if (g1 != g2)
      nextGroup[c++] = i;
  }
  
  count(nextGroup, c);
  for (int j = 0; j < c; j++)
    merge(e[nextGroup[j]].a, e[nextGroup[j]].b);
}

void init() {
  scanf("%ld %ld", &n, &m);
  int a, b, w;
  int k = 0;
  for (int i = 0; i < m; i++) {
    scanf("%ld %ld %ld", &a, &b, &w);
    if (a != b)
      e[k++] = Edge(a, b, w);
  }
  m = k;
}


int main() {
  init();
  kruskal();
  
  int g = group(1);
  for (int i = 1; i < n; i++) {
    if (group(i) != g) {
      ret = 0;
      break;
    }
  }
  
  printf("%lld\n", ret);
  
  return 0;
}
